﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using CoreApplication.Models;
using Syncfusion.DocIO.DLS;
using System.IO;
using Syncfusion.DocIO;
using Microsoft.AspNetCore.Hosting;
using System.Xml;

namespace CoreApplication.Controllers
{
    public class HomeController : Controller
    {
        private readonly IHostingEnvironment _hostingEnvironment;
        public HomeController(IHostingEnvironment hostingEnvironment)
        {
            _hostingEnvironment = hostingEnvironment;
        }

        public async System.Threading.Tasks.Task<ActionResult> Index(string button)
        {
            if (button == null)
                return View();

            // Path of input file.
            string inputPath = _hostingEnvironment.WebRootPath + @"\Data\Template.docx";
            
            // Creating a new document.
            WordDocument document = new WordDocument();
            // Load template          
            FileStream fileStream = new FileStream(inputPath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            document.Open(fileStream, FormatType.Doc);
            fileStream.Dispose();
            fileStream = null;

            string bookmarkName = "tableBookmark";
            //Repalce table inside bookmark.
            ReplaceTable(document, bookmarkName);

            #region Saving document
            MemoryStream memoryStream = new MemoryStream();
            //Save the document into memory stream
            document.Save(memoryStream, FormatType.Docx);
            //Closes the Word document instance
            document.Close();
            memoryStream.Position = 0;
            return File(memoryStream, "application/msword", "Sample.docx");
            #endregion

        }

        /// <summary>
        /// Modify the table inside bookmark.
        /// </summary>
        /// <param name="document"></param>
        /// <param name="bookmarkName"></param>
        public static void ReplaceTable(WordDocument document, string bookmarkName)
        {
            if (string.IsNullOrEmpty(bookmarkName) || document.Bookmarks.FindByName(bookmarkName) == null)
                return;

            //Move to the virtual cursor before the bookmark end location of the bookmark 
            BookmarksNavigator bookmarkNavigator = new BookmarksNavigator(document);
            bookmarkNavigator.MoveToBookmark(bookmarkName);

            //Gets the content for the current bookmark
            TextBodyPart textBodyPart = bookmarkNavigator.GetBookmarkContent();
            WTable table = null;
            var index = 0;
            foreach (Entity entity in textBodyPart.BodyItems)
            {
                if (entity is WTable)
                {
                    table = entity as WTable;
                    break;
                }
                index++;
            }
            if (table == null)
                return;
            //Modify table
            else
            {
                WTableRow row = table.AddRow(true, true);
                IWParagraph paragraph = row.Cells[0].AddParagraph();
                paragraph.AppendText("New cell");
            }
            //Creates new table,
            table = new WTable(document);
            table.ResetCells(2, 3);
            table[0, 0].AddParagraph().AppendText("New Table");
            //Add the new table.
            textBodyPart.BodyItems.Insert(index, table);
            //Add new paragraph
            textBodyPart.BodyItems.Insert(index + 1, new WParagraph(document));
            //Replace the bookmark content
            bookmarkNavigator.ReplaceBookmarkContent(textBodyPart);
        }

        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
  
}
